from .ActivityItem import ActivityItem
from .ActivityLocation import ActivityLocation
from .Address import Address
from .Message import Message
from .Package import Package
from .PackageWeight import PackageWeight
from .ReferenceNumber import ReferenceNumber
from .ReferenceNumberItem import ReferenceNumberItem
from .Response import Response
from .Service import Service
from .Shipment import Shipment
from .ShipmentWeight import ShipmentWeight
from .Shipper import Shipper
from .ShipTo import ShipTo
from .Status import Status
from .StatusCode import StatusCode
from .StatusType import StatusType
from .TrackError import TrackError
from .TrackResponse import TrackResponse
from .TransactionReference import TransactionReference
from .UnitOfMeasurement import UnitOfMeasurement
from .UnitOfMeasurement1 import UnitOfMeasurement1


from dataclasses import dataclass
import xml.etree.ElementTree as ET
import requests, urllib3, xmltodict, json
from UPS_SDK.models.TrackError import TrackError
from UPS_SDK.models.TrackResponse import TrackResponse
urllib3.disable_warnings()

@dataclass
class TrackType:
    ByReferenceNumber = "ReferenceNumber"
    ByTrackingNumber = "TrackingNumber"
    
class UPSApi:
    def __init__(self, access_license_number: str, user_id: str, password: str):
        """
        Create A Ups SDK

        Args:
            access_license_number (str): API Access License Number
            user_id (str): API User ID
            password (str): API Password
        """
        
        self.access_license_number = access_license_number
        self.user_id = user_id
        self.password = password
        self.host = 'https://onlinetools.ups.com/ups.app/xml/Track'

    def create_xml_request(self, value: str, type: TrackType):
        """
        Find Package With ReferenceNumber or TrackingNumber And Returned TrackResponse Object Or Raise TrackError Object

        Args:
            value (str): Reference Number Or Tracking Number
            type (TrackType): Track Type for find, TrackType.ByReferenceNumber or TrackType.ByTrackingNumber

        Raises:
            TrackError: If Tracking Information Not Found Raise Track Error

        Returns:
            TrackResponse: TrackResponse Object
        """

        AccessRequest = ET.Element('AccessRequest')
        ET.SubElement(AccessRequest,'AccessLicenseNumber').text = self.access_license_number
        ET.SubElement(AccessRequest,'UserId').text = self.user_id
        ET.SubElement(AccessRequest,'Password').text = self.password


        TrackToolsRequest = ET.Element('TrackRequest')
        Request = ET.SubElement(TrackToolsRequest,'Request')
        TransactionReference = ET.SubElement(Request,'TransactionReference')
        TrackEL = ET.SubElement(TrackToolsRequest, type)
        
        ET.SubElement(TransactionReference,'CustomerContext').text = 'Customer context'
        ET.SubElement(TransactionReference,'XpciVersion').text = '1.0'
        ET.SubElement(Request,'RequestAction').text = 'Track'
        ET.SubElement(Request,'RequestOption').text = 'activity'
        ET.SubElement(TrackEL, "Value").text = value
        
        _reqString = ET.tostring(AccessRequest)

        tree = ET.ElementTree(ET.fromstring(_reqString))
        root = tree.getroot()

        _QuantunmRequest = ET.tostring(TrackToolsRequest)
        quantunmTree = ET.ElementTree(ET.fromstring(_QuantunmRequest))
        quantRoot = quantunmTree.getroot()
        _XmlRequest: bytes = ET.tostring(root,encoding='utf8', method='xml') + ET.tostring(quantRoot,encoding='utf8', method='xml')
        _XmlRequest = _XmlRequest.decode().replace('\n','')
        
        response = requests.post(self.host, _XmlRequest, verify=False)
        xpars = xmltodict.parse(response.text, dict_constructor=dict)
        json_response = json.loads(json.dumps(xpars))
        track_response: dict = json_response["TrackResponse"]
        
        if "Error" in track_response["Response"].keys():
            raise TrackError(**track_response["Response"]["Error"])
        
        response = TrackResponse(**track_response)
        
        return response
    
    
    
    def get_package(self, reference_number_or_tracking_number: str, track_type: TrackType):
        """
        Get Tracking Information By ReferenceNumber Or TrackingNumber

        Args:
            reference_number_or_tracking_number (str): ReferenceNumber Or TrackingNumber
            track_type (TrackType): TrackType For Find

        Returns:
            TrackResponse: TrackResponse Object Or Raise TrackError
        """
        response = self.create_xml_request(reference_number_or_tracking_number, track_type)
        return response
    
    






from __future__ import annotations
import datetime
from typing import Optional
from pydantic import BaseModel
from . import ActivityLocation, Status, Address

class ActivityItem(BaseModel):
    ActivityLocation: Optional[ActivityLocation]
    Status: Status
    Date: str
    Time: str
    GMTDate: str
    GMTTime: str
    GMTOffset: str
    
    @property
    def description(self) -> str:
        return self.Status.StatusType.Description
    
    @property
    def city(self) -> str:
        return self.ActivityLocation.Address.City
    
    @property
    def address(self) -> Address:
        return self.ActivityLocation.Address
    
    @property
    def status_code(self) -> str:
        return self.Status.StatusCode.Code
    
    @property
    def get_date(self) -> datetime.datetime:
        full_datetime = f"{self.Date} - {self.Time}"
        return datetime.datetime.strptime(full_datetime, "%Y%m%d - %H%M%S")


from __future__ import annotations
from pydantic import BaseModel, Field
from typing import List, Optional
from . import (
    Package, ShipTo, Shipper,
    ReferenceNumberItem, Service, ShipmentWeight)

class Shipment(BaseModel):
    Shipper: Shipper
    ShipTo: ShipTo
    ShipmentWeight: ShipmentWeight
    Service: Service
    ReferenceNumbers: List[ReferenceNumberItem] = Field(None, alias="ReferenceNumber")
    ShipmentIdentificationNumber: str
    PickupDate: str
    ScheduledDeliveryDate: Optional[str]
    Package: Package


from __future__ import annotations
from pydantic import BaseModel, Field
from . import ActivityItem, Message, PackageWeight, ReferenceNumber
from typing import List, Optional

class Package(BaseModel):
    TrackingNumber: str
    DeliveryIndicator: str
    RescheduledDeliveryDate: Optional[str]
    AllActivity: List[ActivityItem] = Field(None, alias="Activity")
    Message: Optional[Message]
    PackageWeight: PackageWeight
    ReferenceNumber: ReferenceNumber
    
    @property
    def get_US_activities(self) -> List[ActivityItem]:
        activities = []
        for activity in self.AllActivity:
            if activity.ActivityLocation is not None and activity.ActivityLocation.Address is not None and activity.ActivityLocation.Address.CountryCode == "US":
                activities.append(activity)
        return activities